------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops
------------------------------------------------------------------------------
-- SellableItemDrops.lua
--
-- Author: Expelliarm5s / August 2022 / All Rights Reserved
--
-- Version 1.4.26
------------------------------------------------------------------------------
-- luacheck: globals sid, ignore 212
-- luacheck: globals DLAPI debugWidgets

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceEvent-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
-- local private = {}
------------------------------------------------------------------------------

addon.METADATA = {
	NAME = GetAddOnMetadata(..., "Title"),
	VERSION = GetAddOnMetadata(..., "Version")
}


------------------------------------------------------------------------------
-- Debug Stuff

function addon:DebugLog(...)
	-- external
	if DLAPI then DLAPI.DebugLog(addonName, ...) end

	-- internal
	-- no more internal DebugLog
end

function addon:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog(res)
		end
	end
end

function addon:ToggleDebug()
	addon.isDebug = not addon.isDebug
	if (not addon.isDebug) then
		sid = nil
		addon:Printf(L["Debug is off"])
		addon:DebugPrintf(L["Debug is off"])
	else
		sid = addon
		addon:Printf(L["Debug is on"])
		addon:DebugPrintf(L["Debug is on"])
	end
end

-- called by AceAddon when Addon is fully loaded
function addon:OnInitialize()
	-- makes Module ABC accessable as addon.ABC
	for module in pairs(addon.modules) do
		addon[module] = addon.modules[module]
	end

	if DLAPI and DLAPI.SetFormat then DLAPI.SetFormat(addonName, "default") end
	addon:DebugPrintf("OnInitialize()")

	-----------------------------------------------------
	-- Debugging
	addon.isDebug = false
	-----------------------------------------------------

	-- loads data and options
	addon.db = AceDB:New(addonName .. "DB", addon.Options.defaults, true)
	AceConfigRegistry:RegisterOptionsTable(addonName, addon.Options.GetOptions)
	local optionsFrame = AceConfigDialog:AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
	addon.Options.frame = optionsFrame

	-- addon flags
	addon.isDemo = false
	addon.isInfight = false
	addon.isClassic = (WOW_PROJECT_ID ~= WOW_PROJECT_MAINLINE)

	addon.ExternalsInfo = ""
	local UJV = GetAddOnMetadata("TheUndermineJournal", "Version")
	if (UJV) then
		if TUJMarketInfo then
			addon.ExternalsInfo = addon.ExternalsInfo .. "The Undermine Journal " .. UJV .. "\n"
		end
	end
	local BBG = GetAddOnMetadata("BootyBayGazette", "Version")
	if (BBG) then
		if TUJMarketInfo then
			addon.ExternalsInfo = addon.ExternalsInfo .. "Booty Bay Gazette " .. BBG .. "\n"
		end
	end
	local TSMV4 = GetAddOnMetadata("TradeSkillMaster", "Version")
	if (TSMV4) then
		if TSM_API and TSM_API.GetCustomPriceValue then
			addon.ExternalsInfo = addon.ExternalsInfo .. "Trade Skill Master 4 " .. TSMV4 .. "\n"
		end
	end
	local AUCV = GetAddOnMetadata("Auctionator", "Version")
	if (AUCV) then
		addon.ExternalsInfo = addon.ExternalsInfo .. "Auctionator " .. AUCV .. "\n"
	end
	if (CanIMogIt) then
		local CIMV = GetAddOnMetadata("CanIMogIt", "Version")
		-- addon:Printf(L["Found %s Version %s."], "Can I Mog It?", CIMV)
		addon.ExternalsInfo = addon.ExternalsInfo .. "Can I Mog It? " .. CIMV .. "\n"
	end
	if AuctionDB and AuctionDB.AHGetAuctionInfoByLink then
		local AHDBV = GetAddOnMetadata("AuctionDB", "Version")
		-- addon:Printf(L["Found %s Version %s."], "AuctionDB", AHDBV)
		addon.ExternalsInfo = addon.ExternalsInfo .. "AuctionDB " .. AHDBV .. "\n"

	end

	-- initialize standard time format
	addon.timeFormat = "%Y-%m-%d %H:%M:%S";
	if (GetLocale() == "deDE") then
		addon.timeFormat = "%d.%m.%Y %H:%M:%S"
	end

	-- initialize chat command
	local chatfunc = function()
		if not addon.GUI.display then
			addon.GUI:Load()
		end
	end
	addon:RegisterChatCommand("sid", chatfunc)
	addon:RegisterChatCommand("siddebug", addon.ToggleDebug)

	-- until thru GUI
	-- addon:RegisterChatCommand("sidsv", addon.ToggleStackedView)

	-- initializing *:Login loop through all modules
	-- done in addon:OnEnable()

	-- initializing *:Logout loop
	addon:RegisterEvent("PLAYER_LOGOUT", function()
		addon:OnLogout()
		end)
end

-- called on PLAYER_LOGOUT
function addon:OnLogout()
	-- loop through all modules calling *:Logout()
	addon:DebugPrintf("Calling Logout() in all modules")
	for module in pairs(addon.modules) do
		if addon.modules[module].Logout then
			addon:DebugPrintf("Call Logout() in module %s", module)
			addon.modules[module]:Logout()
		end
	end

	-- addon:Logout()
end

------------------------------------------------------------------------------
-- Timers

-- loop through all module timers once a second
local function SecTimer()
	local oldDebug = addon.isDebug

	-- to disable debug during timers:
	--	addon.isDebug = false

	for module in pairs(addon.modules) do
		if addon.modules[module].SecTimer then
			addon.modules[module]:SecTimer()
		end
	end

	-- update session summary data and the broker display
	addon.GUI:UpdateSessionData(false)

	-- update session info data some sec. after loading
	-- 	to let the cache fill up the item info data
	if addon.Data.secToUpdate and addon.Data.secToUpdate > 0 then
		addon.Data.secToUpdate = addon.Data.secToUpdate - 1
		if addon.Data.secToUpdate == 10 then
			addon:Printf("Session data updated.")
			addon.GUI:UpdateSessionData(true)
			addon.Data:UpdateSession()
		end
		if addon.Data.secToUpdate == 0 then
			addon:Printf("Item lists updated.")
			addon.GUI:UpdateSessionData(true)
			addon.Data:UpdateSession()
			if addon.GUI.display then
				addon.GUI.container:Reload()
			end
		end
	end

	addon.isDebug = oldDebug
end


-- called by AceAddon on PLAYER_LOGIN
function addon:OnEnable()
	addon:Printf("|cFF33FF99(" .. addon.METADATA.VERSION .. ")|r: " ..
		L["enter /sid for interface"])

	-----------------------------------------------------
	-- Debugging Widgets
	if DLAPI then
		debugWidgets = debugWidgets or {}
		--[[
		debugWidgets.SIDData = {}
		debugWidgets.SIDData.tag = addonName
		debugWidgets.SIDData.DLAPI = DLAPI
		]]
	end
	-----------------------------------------------------

	addon:DebugPrintf("Calling Login() in all modules")
	for module in pairs(addon.modules) do
		if addon.modules[module].Login then
			addon:DebugPrintf("Call Login() in module %s", module)
			addon.modules[module]:Login()
		end
	end

	-- addon:Login()

	-- give every module a chance to do things
	addon.PFCH:Load()
	-- Item: set up item cache etc.
	addon.Item:Login()
	-- Options: initialize global variables
	addon.Options:Load()
	-- LDB: create and initialize LDB broker
	addon.LDB:Load()
	-- Data: load current session data
	addon.Data:Load()
	-- initialize LDB data for loaded session data
	addon:UpdateBroker()
	addon.sectimer = C_Timer.NewTicker(1, SecTimer)

	-- check if the currently used price source is available and check for sanity of options
	addon:CheckSettings()
	-- print info on loaded session data
	if not addon.db.global.noSummary then
		addon.GUI:ShowInfo()
	end

	-- this should not happen, but if there was an error on loading and
	-- initializing old session data, give user a hint to rescue himself
	if not addon.sesItemsLoaded then
		addon:Printf(L["|cffff0000IMPORTANT:|r"] ..
			L["There was an error loading the previous item data. "] ..
			L["Heads up, recover is possible:\n"] ..
			L[" 1. Logout (Character Screen is ok).\n"] ..
			L[" 2. Find your game data directory WTF\\<Account>>\\SaveVariables\n"] ..
			L[" 3. Delete the file "] .. addonName .. L[".lua and rename "] .. addonName .. L[".bak to"] .. addonName .. L[".lua\n"] ..
			L[" 4. Login. "])
	end

	-- enable processing of loot events
	addon.Data:RegisterLootEvents()
end

-- check if the currently used price source is available and check for sanity of options
function addon:CheckSettings()
	local psFallBack = ""
	local mvSourcesList = addon.priceSourcesList or {}
	if not mvSourcesList then
		addon:Printf(L["|cffff0000IMPORTANT:|r"])
		addon:Printf(L["No valid price sources. Please enable UJ or TSM and/or check your price source."])
	else
		local found = false
		for k, _ in pairs(mvSourcesList) do
			-- addon.Data:DebugPrintf("found UJ/TSM Price Source %s/%s", k, v)
			if strlower(k) == strlower(addon.db.global.mvSource) then
				addon.Data:DebugPrintf(" ... using it")
				found = true
				break
			end
		end
		if not found then
			addon:Printf(L["|cffff0000IMPORTANT:|r"])
			addon:Printf(L["Your price source '%s' is currently not usable."],
				mvSourcesList[addon.db.global.mvSource] or addon.db.global.mvSource)
			if addon.db.global.mvSource:match("^TSM:(.+)$") then
				addon:Printf(L["Please enable your TSM modules and/or check if your TSM desktop app is running."])
			elseif addon.db.global.mvSource:match("^UJ:(.+)$") then
				addon:Printf(L["Please enable your The Undermine Journal/Booty Bay Gazette addon."])
			end
			-- if no explicit fallback price source given, TSM uses VendorSell if no AH data is present
			if (psFallBack ~= "") then
				addon:Printf(L["'%s' is used for now."], mvSourcesList[psFallBack] or psFallBack)
				addon.db.global.mvSource = psFallBack
			else
				if addon.db.global.useVendorSellAsDefault then
					addon:Printf(L["Until '%s' is available, 'VendorSell' is used instead."],
						mvSourcesList[addon.db.global.mvSource] or addon.db.global.mvSource)
				end
			end
		end
	end

	-- check for valid noteworthy value
	if not addon.GUI:ToCopper(addon.db.global.minNoteworthy) then
		addon:Printf(L["|cffff0000IMPORTANT:|r"] .. L["Your entry of '%s' for noteworthy items is not usable. It has been reset to 10000g."],
			addon.db.global.minNoteworthy)
		addon.db.global.minNoteworthy = "10000g"
	end
end

-- update LDB data for current session
function addon:UpdateBroker()
	if addon.LDB then
		addon.LDB:UpdateBroker()
	end
end

function addon:ToggleStackedView()
	addon.db.global.stackedView = not addon.db.global.stackedView
	if (not addon.db.global.stackedView) then
		addon:Printf(L["Stacked View is off"])
		if addon.GUI.display then
			addon.GUI.display:SetMagicButtonText(L["S"])
			addon.GUI.display:SetMagicButtonStatusText(L["Switch Stacked View on"])
			addon.GUI.container:Reload()
		end
	else
		addon:Printf(L["Stacked View is on"])
		if addon.GUI.display then
			addon.GUI.display:SetMagicButtonText(L["N"])
			addon.GUI.display:SetMagicButtonStatusText(L["Switch Stacked View off"])
			addon.GUI.container:Reload()
		end
	end
end

-- for testing
local function testitems(item, qty, rarity)
	addon.Data:InsertItem(item, qty, rarity)
end

function addon:sidtest()
	testitems("|cff1eff00|Hitem:15676::::::::111:104::1:1:1705:::|h[Wunderbare Gamaschen der Aurora]|h|r", 1, 0)
	addon.Data:UpdateSession()
	addon.GUI:UpdateSessionData(true)
end


-- EOF
